;******************************************************************************
; Used Hardware:
; AVR Beacon Board with ATtiny2313 processor
; clock rate 8MHz utilizing the internal R/C oscillator
;******************************************************************************
; Copyright (C) February 2008, Ralf Horstmann, www.turbinenheli.de
; Version: 1.1
;
; License Informations:
; This program is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License.
;
; This program is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with this program.  If not, see <http://www.gnu.org/licenses/>
;******************************************************************************
; Changes:
; 03.02.2008 : Implemented a demo mode. Without Jumper the board is running
;			   without a receiver connection. All LED channels are active.
;******************************************************************************
;
; Layout:
; PortB.0 to PortB.7 : LED Outputs, High=Off, Low=On (connect LED to 5V)
; PortD.0 & PortD.1  : Door Sequencer ( future )
; PortD.2 & PortD.3  : RX Channel 1 and Channel 2 Input
; PortD.4 & PortD.5  : special function 1 and special function 2 ( future )
; PortD.6            : Jumper enables Demo-Mode , LOW=On
;
;******************************************************************************

.include "tn2313def.inc"	;Define the chip

; Register
.def Calibrate=r24			;Counter to calibrate R/C
.def RtmpH=r23				;temp. Register for subroutines
.def RtmpL=r22				;temp. Register for subroutines
.def LEDenable1=r21			;LED switch channel 1
.def LEDenable2=r20			;LED switch channel 2
.def loopcnt3=r19			;Counter 3 of delay loop
.def loopcnt2=r18			;Counter 2 of delay loop
.def loopcnt1=r17			;Counter 1 of delay loop
.def address=r16			;EEPROM flash pattern address in mainloop
.def tmpSREG=r7				;register to save SREG

; Definitions
.set NullVal=$5d			;Servo neutral position
.equ MaxVal=(NullVal+$10)	;Servo signal maximum
.equ MinVal=(NullVal-$10)	;Servo signal minimum
.equ INT0in=2				;I/O Port PORTD.2 - INT0 - CHANNEL1
.equ INT1in=3				;I/O Port PORTD.3 - INT1 - CHANNEL2
.equ JUMPER=6				;I/O Port PORTD.6 - Jumper

;******************************************************************************
; Interrupts
;******************************************************************************
.cseg
INITINT:
.org $0000 
	rjmp INIT				;Reset extern or Watchdog
	rjmp CHANNEL1			;External Interrupt INT0 - Channel 1
	rjmp CHANNEL2			;External Interrupt INT1 - Channel 2
	reti ;rjmp Timer1_capt	;Timer 1 capture Mode
	reti ;rjmp Timer1_comp	;Timer 1 compare Mode
	rjmp NO_SERVO_SIGNAL	;Timer 1 overflow
	reti ;rjmp Timer0_over	;Timer 0 overflow
	reti ;rjmp UART_rxc		;Serial RX complete Handler
	reti ;rjmp UART_dre		;Serial UDR empty Handler
	reti ;rjmp UART_txc		;Serial TX complete Handler
	reti ;rjmp ANA_comp		;Analog comperator
	reti ;rjmp PCINT		;PCINT
	reti ;rjmp TIMER1_COMPB ;TIMER1 COMPB
	reti ;rjmp TIMER0_COMPA ;TIMER0 COMPA
	reti ;rjmp TIMER0_COMPB ;TIMER0 COMPB
	reti ;rjmp USI_START	;USI START
	reti ;rjmp USI_OVERFLOW	;USI OVERFLOW
	reti ;rjmp EE_READY		;EE READY
	reti ;rjmp WDT_OVERFLOW	;WDT OVERFLOW

;******************************************************************************
;Initialization
;******************************************************************************
INIT:
	cli						;Disable all interrupts	
	ldi RtmpH,low(RAMEND)	;Set Stackpointer to end of RAM, Stack works backwards
	out SPL,RtmpH

	;configure outputs
    ldi RtmpH,0b11111111	;Pin PB0 - PB7 = output (LED's)
    out DDRB,RtmpH			;Port B -> output (LED's for flash pattern)
	out PORTB,RtmpH			;Port B -> all Pins high = LEDs off
	ldi RtmpH,0b11110011	;Pin PD0,PD1,PD4,PD5=output (Servo connectors)
	out DDRD,RtmpH			;Port D -> Servos + Interrupt inputs of PD2, PD3 + Config Jumper (PD6)
	ldi RtmpH,0b00000000	;INT0 Pullup disable
	out PORTD,RtmpH

	;Timer0 configuration
	ldi RtmpH,0b00000010	;Timer0 prescaling 1/8 bei 4MHz
	out TCCR0,RtmpH

	;Timer 1 configuration
	;Puls length between 800us and 2200us, neutral at 1500us
	;
	;4 MHz and 1/64 divider (1 counter = 16us) :
	;800us ~ 50 cycles - HB:$00 LB:$32
	;1500us ~ 93 cycles -  HB:$00 LB:$5d
	;2200us ~ 137 cycles - HB:$00 LB:$89
	;
	;8 MHz and 1/64 divider (1 counter = 8us) :
	;800us ~ 100 cycles - HB:$00 LB:$64
	;1500us ~ 187 cycles -  HB:$00 LB:$bb
	;2200us ~ 275 cycles - HB:$01 LB:$13
	ldi RtmpH,0b00000011	;Timer1 prescaling 1/64 and start Timer
	sbic PIND,JUMPER      	;Demo if Jumper set
	ldi RtmpH,$00 			;Timer1 stopped if Demo Mode
	out TCCR1B,RtmpH	
	ldi RtmpH,$00			;Timer1 disconnect OC1 + PWM
	out TCCR1A,RtmpH
	out TCNT1L,RtmpH		;Timer1 init low value
	out TCNT1H,RtmpH		;Timer1 init high value
	out TCNT0,RtmpH			;Timer0 init value
	ldi LEDenable1,0b11111111;Deaktivating LEDs of CHANNEL1
	ldi LEDenable2,0b11111111;Deaktivating LEDs of CHANNEL2
	ldi Calibrate,$10		;analysing 16 Servo pulses

	;Watchdog configuration
	wdr
	in RtmpH,WDTCSR
	ori RtmpH,(1<<WDCE) | (1<<WDE)
	out WDTCSR,RtmpH
	ldi RtmpH,(1<<WDE) | (1<<WDP2) | (1<<WDP1) | (1<<WDP0)	;Enable Watchdog, Time 2s
	out WDTCSR,RtmpH		;Start Watchdog

	;Interrupt configuration
	ldi RtmpH,0b10000010	;Timer0 + Timer1 overflow interrupt enabled		
	out TIMSK,RtmpH			
	ldi RtmpH,0b11000000	;enable INT0 + INT1
	out GIMSK,RtmpH		
	ldi RtmpH,0b00001111	;INT1 rising edge, INT0 rising edge
	out MCUCR,RtmpH		
	ldi RtmpH,0b11000000	;clear initial interrupts
	out GIFR,RtmpH
    sei						;Global Interrupts aktivieren

;******************************************************************************
;Main program
;******************************************************************************
START:
	ldi address,FLASHPATTERM;EEPROM address of flash pattern
MAINLOOP:
	wdr						;Watchdog Reset
	and LEDenable1,LEDenable2;combine LED mask of channel 1 and channel 2
	sbic PIND,JUMPER		;Demo if Jumper set
	ldi LEDenable1,0b00000000;activate all channels, ignore transmitter
	out EEAR,address		;address EEPROM
	sbi EECR,EERE			;start read
	in RtmpH,EEDR			;get value from EEPROM
	or RtmpH,LEDenable1		;calculate LEDs
	out PORTB,RtmpH         ;switch LEDs
	rcall Delay50ms         ;wait 50ms
	inc address				;increase EEPROM address
	cpi address,FLASHPATTERM+20;read 20 values ?
	brlo MAINLOOP			;no -> continue with flash pattern
	rjmp START				;yes -> restart flash pattern
			
;******************************************************************************
; subroutines and interrupt routines
;******************************************************************************
;delay loop 50ms
Delay50ms:        			;wait 400000 cycles ~ 50ms
;delaying 399999 cycles:
	ldi  loopcnt1,$97
LOOP0_50ms:
	ldi  loopcnt2,$06
LOOP1_50ms:
	ldi  loopcnt3,$92
LOOP2_50ms:
	dec  loopcnt3
	brne LOOP2_50ms
	dec  loopcnt2
	brne LOOP1_50ms
	dec  loopcnt1
	brne LOOP0_50ms
	ret

;******************************************************************************
; Servo Input CHANNEL 1
;******************************************************************************
.cseg
CHANNEL1:
	in tmpSREG,SREG			;secure SREG
	push RtmpL				;secure temp. Register
	push RtmpH
	push address
	sbis PIND,INT0in		;skip on raising INT0, skip
	rjmp INT0_fall			;measure the signal

INT0_rise:
	in  RtmpH,MCUCR			;get bits in MCUCR
	andi RtmpH,0b11111110	;Set Bits ISC00,ISC01
	out MCUCR,RtmpH			;INT0 falling edge
	rjmp End_INT_general	;Interrupt End

INT0_fall:					;wait for falling edge and get servo value
	in RtmpH,TCNT1H			;secure counter value of CHANNEL1
	in RtmpL,TCNT1L			;secure counter value of CHANNEL1
	clc						;Clear Carry
	ror RtmpH				;divide measurement by 2 (at 8MHz chip clock only)
	ror RtmpL				;

;	cpi Calibrate,$0		;calibration done ?
;	brne INT0_calibrate		;if counter <> 0
	
	ldi address,CHANNEL1MASK	;address CHANNEL 1 Mask
	cpi RtmpL,MaxVal		;compare with maximum
	brlo INT0_chkNeutral	;value is smaller than MaxVal
	;top third
	rjmp End_INT0_fall

INT0_chkNeutral:
	inc address				;increase EEPROM address
	cpi RtmpL,MinVal		;compare with value for neutral position
	brlo INT0_OFF			;value smaller than MinVal
	;neutral third
	rjmp End_INT0_fall

INT0_OFF:					;Signal too long
	;lower third
	inc address				;increase EEPROM address

End_INT0_fall:
	out EEAR,address		;address EEPROM
	sbi EECR,EERE			;start read
	in LEDenable1,EEDR		;get value from EEPROM (CHANNEL1Maske)
	in  RtmpH,MCUCR			;set INT0 to raising edge
	ori RtmpH,0b00000011	;Set Bits ISC00, ISC01
	out MCUCR,RtmpH			;INT0 rising edge
	rjmp End_INT_general	;Interrupt End

;******************************************************************************
;INT0_calibrate:
;	dec Calibrate			;decrease calibration counter
;	brne End_INT0_fall		;return if not 0
;	ldi Ktmp,$20
;	out EEAR,Ktmp			;address EEPROM
;	out EEDR,RTmpL			;write Lowbyte
;	sbi EECR,2				;write
;	sbi EECR,1				;write
;INT0_calibrate_wait:
;	sbic EECR,EEWE
;	rjmp INT0_calibrate_wait
;	ldi Ktmp,$21
;	out EEAR,KTmp			;address EEPROM
;	out EEDR,RTmpH			;write Highbyte
;	sbi EECR,2				;write
;	sbi EECR,1				;write
;	rjmp End_INT0_fall

;******************************************************************************
; Servo Input CHANNEL 2
;******************************************************************************
.cseg
CHANNEL2:
	in tmpSREG,SREG			;secure SREG
	push RtmpL				;secure temp. Register
	push RtmpH
	push address
	sbis PIND,INT1in		;skip if raising edge of INT1
	rjmp INT1_fall			;measure signal

INT1_rise:
	in  RtmpH,MCUCR			;get Bits of MCUCR
	andi RtmpH,0b11111011	;Set Bits ISC10,ISC11
	out MCUCR,RtmpH			;INT1 falling edge
	rjmp End_INT_general	;Interrupt End

INT1_fall:
	in RtmpH,TCNT1H			;secure counter of CHANNEL2
	in RtmpL,TCNT1L			;secure counter of CHANNEL2
	clc						;Clear Carry
	ror RtmpH				;divide value by 2 (only at 8MHz chip clock)
	ror RtmpL				;

	ldi address,CHANNEL2MASK	;address CHANNEL 2 Mask
	cpi RtmpL,MaxVal		;compare with maximum value
	brlo INT1_chkNeutral	;smaller than MaxVal
	;top third
	rjmp End_INT1_fall

INT1_chkNeutral:
	inc address				;increase EEPROM address 
	cpi RtmpL,MinVal		;compare with neutrale value
	brlo INT1_OFF			;smaller than MinVal
	;neutral third
	rjmp End_INT1_fall

INT1_OFF:					;Signal too long
	;lower third
	inc address				;increase EEPROM address

End_INT1_fall:
	out EEAR,address		;address EEPROM
	sbi EECR,EERE			;start read
	in LEDenable2,EEDR		;get value from EEPROM (CHANNEL1Maske)
	in  RtmpH,MCUCR			;change INT1 to raising edge
	ori RtmpH,0b00001100	;Set Bits ISC10, ISC11
	out MCUCR,RtmpH			;INT1 rising edge

;******************************************************************************
;Interrupt End for CHANNEL1 and CHANNEL2
;******************************************************************************
End_INT_general:
	clr RtmpH				;reset Timer1
	out TCNT1H,RtmpH
	out TCNT1L,RtmpH
	pop address				;restore temp Register
	pop RtmpH
	pop RtmpL
	out SREG,tmpSREG		;recover SREG
	reti

;******************************************************************************
;At Timer1 overflow, there hasn't been a receiver signal for approx 1 Sec.,
;about 20 signals were missed. Needed for doorsequencer handling in the future
;******************************************************************************
NO_SERVO_SIGNAL:		;no receiver signal
	in tmpSREG,SREG		;securing SREG
NO_SIGNAL_LOOP:
	rjmp NO_SIGNAL_LOOP	;wait for reset by Watchdog

;******************************************************************************
;flash pattern definition, clock rate 0,05 s
;******************************************************************************
.eseg
FLASHPATTERM:
	.db 0b11111000,0b11111000
	.db 0b11111110,0b11111110
	.db 0b11111000,0b11111000
	.db 0b11111110,0b11111110
	.db 0b11111110,0b11111110
	.db 0b11111110,0b11111110
	.db 0b11111110,0b11111110
	.db 0b11111110,0b11111110
	.db 0b11111110,0b11111110
	.db 0b11111110,0b11111110
CHANNEL1MASK:
	.db 0b11111000,0b11111000,0b11111111
CHANNEL2MASK:
	.db 0b11111111,0b11111111,0b11111111
;program end
.exit
